//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

void* linkedchannellist = NULL;

struct weircd_channel* cdat(CHANNEL chan) {
	return linkedlist_GetData(chan);
}
CHANNEL cyclechan;
void chan_CycleInit() {
	cyclechan = linkedchannellist;
}
CHANNEL chan_DoCycle() {
	if (cyclechan == NULL) {return NULL;}
	void* retval = cyclechan;
	cyclechan = linkedlist_NextItem(cyclechan);
	return retval;
}
CHANNEL chan_FindByName(const char* channame) {
	chan_CycleInit();
	CHANNEL chan = chan_DoCycle();
	while (chan != NOCHAN) {
		if (strcasecmp(cdat(chan)->name,channame) == 0) {
			return chan;
		}
		chan = chan_DoCycle();
	}
	return NOCHAN;
}
CHANNEL chan_Add(const char* channame) {
	CHANNEL chan = chan_FindByName(channame);
	if (chan != NOCHAN) {return chan;}
	struct weircd_channel cdata;
	memset(&cdata,0,sizeof(cdata));
	strncpy(cdata.name, channame, MAXCHANNELNAMELEN);
	cdata.name[MAXCHANNELNAMELEN] = 0;
	cdata.creationtime = ts();
	void* item = linkedlist_Append(&linkedchannellist,&cdata,sizeof(cdata));
	if (item == NULL) {return NOCHAN;}
	return item;
}
void chan_FreeBan(CHANNEL chan, int banid) {
	if (chan == NOCHAN) {return;}
	if (cdat(chan)->ban[banid] == NULL) {return;}
	if (cdat(chan)->ban[banid]->actor != NULL) {
		free(cdat(chan)->ban[banid]->actor);
	}
	if (cdat(chan)->ban[banid]->mask != NULL) {
		free(cdat(chan)->ban[banid]->mask);
	}
	free(cdat(chan)->ban[banid]);
}
void chan_FreeInvite(CHANNEL chan, int inviteid) {
	if (chan == NOCHAN) {return;}
	if (cdat(chan)->invite[inviteid] == NULL) {return;}
	free(cdat(chan)->invite[inviteid]);
}
void chan_AddInviteEntry(CHANNEL chan, int entryid, USER target, unsigned long time) {
	cdat(chan)->invite[entryid] = malloc(sizeof(struct weircd_channelinvite));
	if (cdat(chan)->invite[entryid] == NULL) {return;}
	memset(cdat(chan)->invite[entryid],0,sizeof(struct weircd_channelinvite));
	cdat(chan)->invite[entryid]->target = target;
	cdat(chan)->invite[entryid]->time = time;
	//printf("%s: Added invite entry %d [%s]\n",cdat(chan)->name,entryid,udat(cdat(chan)->invite[entryid]->target)->nickname);
}
void chan_AddInvite(CHANNEL chan, USER target, unsigned long time) {
	int r = 0; /* check if that entry is already there: if yes, update the time */
	while (r < MAXINVITESPERCHAN) {
		if (cdat(chan)->invite[r] != NULL) {
			if (cdat(chan)->invite[r]->target == target) {
				cdat(chan)->invite[r]->time = time;
				return;
			}
		}
		r++;
	}
	/* find an empty entry. if there is one, use it */
	r = 0;
	while (r < MAXINVITESPERCHAN) {
		if (cdat(chan)->invite[r] == NULL) {
			chan_AddInviteEntry(chan, r, target, time);
			return;
		}
		r++;
	}
	/* ok, we need to overwrite the oldest entry as there is apparently no empty entry left */
	int oldest_id = -1;unsigned long oldest_time;
	while (r < MAXINVITESPERCHAN) {
		if (cdat(chan)->invite[r] != NULL) {
			if (cdat(chan)->invite[r]->time < oldest_time || oldest_id < 0) {
				oldest_id = r;
				oldest_time = cdat(chan)->invite[r]->time;
			}
		}
		r++;
	}
	if (oldest_id < 0) {return;}
	/* set the new properties of the invite line */
	free(cdat(chan)->invite[oldest_id]->target);
	cdat(chan)->invite[oldest_id]->target = malloc(strlen(target)+1);
	if (cdat(chan)->invite[oldest_id]->target == NULL) {
		free(cdat(chan)->invite[oldest_id]);
		cdat(chan)->invite[oldest_id] = 0;return;
	}
	strcpy(cdat(chan)->invite[oldest_id]->target, target);
	cdat(chan)->invite[oldest_id]->time = time;
}
void chan_RemoveOldInvites(CHANNEL chan) {
	int r = 0;
	while (r < MAXINVITESPERCHAN) {
		if (cdat(chan)->invite[r] != NULL) {
			if (cdat(chan)->invite[r]->time + INVITELEN < ts()) {
				//printf("%s: Expiring invite entry %d [%s]\n", cdat(chan)->name,r,udat(cdat(chan)->invite[r]->target)->nickname);
				chan_FreeInvite(chan,r);
				cdat(chan)->invite[r] = NULL;
			}
		}
		r++;
	}
}
void chan_Delete(CHANNEL id) {
	if (id == NOCHAN) {return;}
	int r = 0;
	while (r < MAXBANSPERCHAN) {
		if (cdat(id)->ban[r] != NULL) {
			chan_FreeBan(id,r);
		}
		r++;
	}
	r = 0;
	while (r < MAXINVITESPERCHAN) {
		if (cdat(id)->invite[r] != NULL) {
			chan_FreeInvite(id,r);
		}
		r++;
	}
	if (cdat(id)->password != NULL) {
		free(cdat(id)->password);
	}
	linkedlist_RemoveItem(&linkedchannellist,id);
}
