//    weircdservices module
//    Copyright (C) 2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "../weircd_api.h"
#include "../weircd_api_fakeusers.h"
#include "../weircd_api_services.h"

//#include "md5.h"
char md5result[50];
#include "md5.c"

/* Preface: some options */

char host[200];
int nicknameregisterwaittime;
int dolimitchanchars;
int forcerenameseconds;

void privmsg_nickserv(const char* user, const char* msg);
void privmsg_chanserv(const char* user, const char* msg);
void privmsg_operserv(const char* user, const char* msg);
void services_chanservjoining();
int opendbs();
void closedbs();

/* Those are needed for processing auto op/auto voice/auto half op on user join */
int services_channelregistered(const char* channel);
int services_chanid(const char* channel);
void services_setchanneltopic(const char* channel, const char* topic);
const char* services_getchanflags(const char* username, int chanid);

/* Section 1: InfoQuery function providing info about the module*/

int major = 1;
int minor = 2;
int release = 0;
void module_InfoQuery() {
	weircd_infoq_ReportName("WeIRCd Services");
	weircd_infoq_ReportVersion(major,minor,release);
	weircd_infoq_ReportComment("The official built-in services");
	weircd_infoq_ReportAuthor("Jonas Thiem");
	weircd_infoq_ReportServicesFunctionality();
}

/* Section 2: pre parsing config options */

int optionnickwaittime;
int optionhost;
int optionlimitchanchars;
int optionforcerename;
void module_ServicesCOptionPreCheck_Begin() {
	//reset the option switches (to indicate no options have been set yet)
	optionnickwaittime = -1;
	optionhost = 0;
	optionlimitchanchars = 0;
	optionforcerename = 0;
}
int module_ServicesCOptionPreCheck(const char* option, const char* value) {
	//evaluate the config option and return 0 on success
	//set the option switch of that option to 1 to indicate it has been set already
	//printf("PRECHECK %s=>%s\n",option,value);
	if (strcasecmp(option,"host") == 0) {
		if (optionhost == 1) {return -1;}
		if (strlen(value) <= 0 || strlen(value) >= 130) {return -1;}
		if (charpos(value,' ') >= 0) {return -1;}
		if (charpos(value,'!') >= 0) {return -1;}
		if (charpos(value,'@') >= 0) {return -1;}
		if (charpos(value,',') >= 0) {return -1;}
		optionhost = 1;
		return 0;
	}
	if (strcasecmp(option,"forcerename") == 0) {
		if (optionforcerename == 1) {return -1;}
		if (issimplenumber(value) != 0) {return -1;}
		if (strlen(value) < 0 || strlen(value) > 6) {return -1;}
		if (atoi(value) < 0) {return -1;}
		optionforcerename = 1;
		return 0;
	}
	if (strcasecmp(option,"nickwaittime") == 0) {
		if (optionnickwaittime == 1) {return -1;}
		if (issimplenumber(value) != 0) {return -1;}
		if (strlen(value) < 0 || strlen(value) > 6) {return -1;}
		if (atoi(value) < 0) {return -1;}
		optionnickwaittime = 1;
		return 0;
	}
	if (strcasecmp(option,"limitchanchars") == 0) {
		if (optionlimitchanchars == 1) {return -1;}
		if (strcasecmp(value,"true") != 0 && strcasecmp(value,"false") != 0) {
			return -1;
		}
		optionlimitchanchars = 1;
		return 0;
	}
	return -1;
}
int module_ServicesCOptionPreCheck_End() {
	//check if config options are missing
	if (optionnickwaittime < 0 || optionhost == 0 || optionlimitchanchars == 0 || optionforcerename == 0) {printf("missing %d|%d|%d|%d\n",optionnickwaittime,optionhost,optionlimitchanchars,optionforcerename);return -1;}
	return 0;
}

/* Section 3: Fake user api handling */

int chanserv = -1;
int nickserv = -1;
int operserv = -1;
int global = -1;

void LoginBot(int handle, const char* nick) {
	if (weircd_services_LoginFakeuserAsServiceBot(handle, nick, "services", host, "WeIRCd built-in services") < 0) {
		//printf("%s login failed\n",nick);
	}
}

void CreateBotsIfMissing() {
	if (chanserv < 0) {
		chanserv = weircd_fakeuser_Create();
		if (chanserv >= 0) {
			LoginBot(chanserv, "ChanServ");
		}
	}
	if (nickserv < 0) {
		nickserv = weircd_fakeuser_Create();
		LoginBot(nickserv, "NickServ");
	}
	if (operserv < 0) {
		operserv = weircd_fakeuser_Create();
		LoginBot(operserv, "OperServ");
	}
	if (global < 0) {
		global = weircd_fakeuser_Create();
		LoginBot(global, "Global");
	}
}

char secondvalbuf[400];
const char* GetSecondValue(const char* str) {
	strncpy(secondvalbuf,str,399);secondvalbuf[399] = 0;
	trimleft(secondvalbuf);
	if (charpos(secondvalbuf,' ') <= 0) {strcpy(secondvalbuf,"");return secondvalbuf;}
	cut(secondvalbuf,charpos(secondvalbuf,' ')+1);
	trimleft(secondvalbuf);
	cutonfirstspace(secondvalbuf);
	return secondvalbuf;
}
void CutTwice(char* str) {
	trimleft(str);
	if (charpos(str,' ') <= 0) {strcpy(str,"");return;}
	cut(str,charpos(str,' ')+1);
	trimleft(str);
	if (charpos(str,' ') <= 0) {strcpy(str,"");return;}
	cut(str,charpos(str,' ')+1);
	trimleft(str);
}
void CutThree(char* str) {
	trimleft(str);
	if (charpos(str,' ') <= 0) {strcpy(str,"");return;}
	cut(str,charpos(str,' ')+1);
	trimleft(str);
	if (charpos(str,' ') <= 0) {strcpy(str,"");return;}
	cut(str,charpos(str,' ')+1);
	trimleft(str);
	if (charpos(str,' ') <= 0) {strcpy(str,"");return;}
	cut(str,charpos(str,' ')+1);
	trimleft(str);
}
//char notready[] = "Sadly the services are \002not ready yet\002. But as this bot proves, they are on their way back. Please stay tuned.";
void msguser(int handle,const char* user, const char* msg) {
	char send[513];
	snprintf(send,512,"NOTICE %s :%s",user,msg);send[512] = 0;
	weircd_fakeuser_Send(handle,send);
}
int services_userregistered(const char* nickname);
void examineuser(const char* user) {
	char msg[500];
	if (services_userregistered(user) == 0) {
		const char* acc = weircd_user_GetLoginAccountName(user);
		if (acc != NULL) {
			if (strcasecmp(acc,user) == 0) {
				return;
			}
		}
		snprintf(msg,499,"*** Your nickname is \002registered\002. Please change your nickname or login using /msg NickServ identify <password>.");
		msg[499] = 0;
		msguser(nickserv, user, msg);
		if (forcerenameseconds > 0) {
			snprintf(msg,499,"If you do not login, you will be force-renamed after \002%d seconds\002.", forcerenameseconds);
			msg[499] = 0;
			msguser(nickserv, user, msg);
		}
	}
}
void unexamineuser(const char* user) {
	//stop force rename countdown because user has quit
}
void module_FakeUserRecv(int handle, const char* data) {
	char* l = alloca(strlen(data)+1);
	char msg[500];
	strcpy(l,data);
	//printf(">> [%d] %s\n",handle,data);
	const char* p = GetSecondValue(l);
	if (issimplenumber(p) == 0) {
		//numerics
		int i = atoi(p);
		if (i == 650 && handle == nickserv) {
			CutThree(l);
			cutonfirstspace(l);
			if (strcasecmp(l,"nickserv") == 0 || strcasecmp(l,"chanserv") == 0 || strcasecmp(l,"operserv") == 0 || strcasecmp(l,"global") == 0) {return;}
			snprintf(msg,499,"Welcome to \002%s\002. We provide services to enable the registration of nicknames and channels! For details, type \002/msg NickServ help\002 and \002/msg ChanServ help\002.", weircd_GetNetworkName());
			msg[499] = 0;
			msguser(nickserv, l, msg);
			examineuser(l);
			return;
		}
	}else{
		char actor[100];
		strncpy(actor,l,99);actor[99] = 0;
		if (strstw(actor,":") == 0) {cut(actor,1);}
		if (charpos(actor,'!') > 0) {actor[charpos(actor,'!')] = 0;}
		if (strcasecmp(p, "join") == 0 && handle == chanserv) {
			CutTwice(l);trimleft(l);
			if (strstw(l,":") == 0) {cut(l,1);trimleft(l);}
			/* Process auto op etc: */
			const char* acc = weircd_user_GetLoginAccountName(actor);
			if (acc != NULL) {//check if user is registered
				if (services_channelregistered(l) == 0) {//check if channel is registered
					/* Now check channel's flags list entry for that user */
					int id = services_chanid(l);
					if (id >= 0) {
						const char* flags = services_getchanflags(acc, id);
						if (flags != NULL) {
							char setflags[4] = "";
							if (charpos(flags,'O') >= 0) {
								setflags[strlen(setflags)+1] = 0;
								setflags[strlen(setflags)] = 'o';
							}
							if (charpos(flags,'H') >= 0) {
								setflags[strlen(setflags)+1] = 0;
								setflags[strlen(setflags)] = 'h';
							}
							if (charpos(flags,'V') >= 0) {
								setflags[strlen(setflags)+1] = 0;
								setflags[strlen(setflags)] = 'v';
							}
							if (strlen(setflags) > 0) {
								snprintf(msg,499,"MODE %s +%s",l,setflags);
								msg[499] = 0;
								int count = 0;
								while (count < strlen(setflags)) {
									stradd(msg," ",499);
									stradd(msg,actor,499);
									count++;
								}
								weircd_fakeuser_Send(chanserv,msg);
							}
						}
					}
				}
			}
		}
		if (strcasecmp(p, "privmsg") == 0) {
			CutTwice(l);
			char tg[50];
			if (charpos(l,':') <= 0) {return;}
			strncpy(tg,l,49);tg[49] = 0;trimright(tg);
			if (charpos(tg,':') >= 0) {tg[charpos(tg,':')] = 0;trimright(tg);}
			cut(l,charpos(l,':')+1);trimright(l);trimleft(l);
			if (strlen(l) > 0) {
				if (l[0] == 1 && l[strlen(l)-1] == 1) {//ctcp
					cut(l,1);if (strlen(l) > 0) {l[strlen(l)-1] = 0;}
					trimleft(l);trimright(l);
					if (strcasecmp(l,"finger") == 0 || strstw(l,"finger") == 0) {
						msguser(handle, actor, "\001FINGER Please use a regular WHOIS or the VERSION query to get extended information.\001");
						return;
					}
					if (strcasecmp(l,"version") == 0 || strstw(l,"version ") == 0) {
						snprintf(msg,499,"\001VERSION WeIRCd built-in services v%d.%d.%d - http://eloxoph.com/weircd/\001",major,minor,release);
						msg[499] = 0;
						msguser(handle, actor, msg);
						return;
					}
					if (strcasecmp(l,"clientinfo") == 0 || strstw(l,"clientinfo") == 0) {
						msguser(handle, actor, "\001CLIENTINFO CLIENTINFO PING VERSION\001");
						return;
					}
					if (strstw(l,"ping ") == 0 || strcasecmp(l,"ping") == 0) {
						cut(l,strlen("PING "));trimleft(l);cutonfirstspace(l);
						snprintf(msg,499,"\001PING %s\001",l);
						msg[499] = 0;
						msguser(handle, actor, msg);
						return;
					}
					if (strcasecmp(l,"userinfo") == 0 || strstw(l,"userinfo") == 0) {
						msguser(handle, actor, "\001USERINFO Not much userinfo to give for a bot.\001");
						return;
					}
					return;
				}
			}
			if (handle == chanserv && strcasecmp(tg,"chanserv") == 0) {
				privmsg_chanserv(actor,l);
			}
			if (handle == nickserv && strcasecmp(tg,"nickserv") == 0) {
				privmsg_nickserv(actor,l);
			}
			if (handle == operserv && strcasecmp(tg,"operserv") == 0) {
				privmsg_operserv(actor,l);
			}
			return;
		}
		if (strcasecmp(p,"NICK") == 0 && handle == nickserv) {
			CutTwice(l);
			trimleft(l);
			if (strstw(l,":") == 0) {cut(l,1);trimleft(l);}
			cutonfirstspace(l);
			if (strcasecmp(l,actor) != 0) {
				examineuser(l);
			}
		}
		if (strcasecmp(p,"QUIT") == 0 && handle == nickserv) {
			unexamineuser(actor);
		}
		if (strcasecmp(p,"TOPIC") == 0 && handle == chanserv) {
			CutTwice(l);
			char tg[50];
			if (charpos(l,':') <= 0) {return;}
			strncpy(tg,l,49);tg[49] = 0;trimright(tg);
			if (charpos(tg,':') >= 0) {tg[charpos(tg,':')] = 0;trimright(tg);}
			cut(l,charpos(l,':')+1);trimright(l);trimleft(l);
			services_setchanneltopic(tg,l);
		}
	}
}

/* Section 4: Services startup & shutdown */

int module_ServicesStartup(int ocount, const char** oname, const char** ovalue) {
	//printf("SERVICES: starting up services\n");
	int r = 0;
	while (r < ocount) {
		if (strcasecmp(oname[r],"host") == 0) {
			strncpy(host,ovalue[r],199);host[199] = 0;
		}
		if (strcasecmp(oname[r],"nickwaittime") == 0) {
			nicknameregisterwaittime = atoi(ovalue[r]);
		}
		if (strcasecmp(oname[r],"limitchanchars") == 0) {
			if (strcasecmp(ovalue[r],"true") == 0) {
				//printf("option: limit chan chars!\n");
				dolimitchanchars = 1;
			}else{
				dolimitchanchars = 0;
				//printf("option: do not limit chan chars!\n");
			}
		}
		if (strcasecmp(oname[r],"forcerename") == 0) {
			forcerenameseconds = atoi(ovalue[r]);
		}
		r++;
	}
	//printf("now trying to open dbs\n");
	if (opendbs() != 0) {
		return -1;
	}
	CreateBotsIfMissing();
	services_chanservjoining();
	//printf("all done\n");
	return 0;
}
void module_ServicesShutdown() {
	chanserv = -1;
	closedbs();
	//printf("SERVICES: services are shutdown\n");
}
int module_NewOptionSet(int ocount, const char** oname, const char** ovalue) {
	int r = 0;
	while (r < ocount) {
		if (strcasecmp(oname[r],"host") == 0) {
			if (strcasecmp(host,ovalue[r]) != 0) {
				strncpy(host,ovalue[r],199);host[199] = 0;
				//host differs, we need to change it for all our bots
				
			}
		}
		if (strcasecmp(oname[r],"nickwaittime") == 0) {
			nicknameregisterwaittime = atoi(ovalue[r]);
		}
		if (strcasecmp(oname[r],"limitchanchars") == 0) {
			if (strcasecmp(ovalue[r],"true") == 0) {
				dolimitchanchars = 1;
			}else{
				dolimitchanchars = 0;
			}
		}
		r++;
	}
	return 0;
}

/* Section 5: DB stuff */

#include "weircdservices_db.c"

/* Section 6: Channel flags logic */

#include "weircdservices_channelflags.c"

/* Section 7: Proceed messages */

#include "weircdservices_chanservmsghandling.c"
#include "weircdservices_nickservmsghandling.c"
#include "weircdservices_operservmsghandling.c"




