//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

// This implements modules/weircd_api.h functions

int modulebroken = 0;
char* reportedname = NULL;
char* reportedcomment = NULL;
char* reportedauthors = NULL;
int servicesfunc = 0;
int linkingfunc = 0;
int linkingwithsid = 0;
int vmajor = 0;int vminor = 0;int vrevision = 0;

void weircdapi_PrepareInfoQuery(const char* modname) {
	if (reportedname != NULL) {free(reportedname);reportedname = NULL;}
	if (reportedcomment != NULL) {free(reportedcomment);reportedcomment = NULL;}
	if (reportedauthors != NULL) {free(reportedauthors);reportedauthors = NULL;}
	modulebroken = 0;
	servicesfunc = 0;
	linkingfunc = 0;
	linkingwithsid = 0;
	vmajor = 0;vminor = 0;vrevision = 0;
	module_Switched(modname);
	module_SetArea(MODAREA_INFOQ);
}
void weircdapi_CloseInfoQuery() {
	if (modulebroken > 0) {return;}
	if (module_AccessAllowed(MODAREA_INFOQ) != 0) {return;}
	if (reportedname == NULL) {
		modulebroken = MODULEFAIL_NoName;
		return;
	}
	if (vmajor <= 0 && vminor <= 0 && vrevision <= 0) {
		modulebroken = MODULEFAIL_NoVersion;
		return;
	}
}
int weircdapi_IsModuleBroken() {
	return modulebroken;
}
const char* weircdapi_GetName() {return reportedname;}
void weircdapi_SetInfo(struct moduleinfo* inf) {
	inf->services = servicesfunc;
	inf->linking = linkingfunc;
	inf->major = vmajor;inf->minor = vminor;inf->revision = vrevision;
	inf->linkingwithsid = linkingwithsid;
}
void weircd_infoq_ReportName(const char* name) {
	if (modulebroken > 0) {return;}
	if (module_AccessAllowed(MODAREA_INFOQ) != 0) {return;}
	if (strlen(name) <= 0) {modulebroken = MODULEFAIL_NameInvalid;return;}
	if (strlen(name) > 50) {modulebroken = MODULEFAIL_NameInvalid;return;}
	unsigned int r = 0;int previousspace = 1;
	while (r < strlen(name)) {
		if ((name[r] >= 'a' && name[r] <= 'z') || (name[r] >= 'A' && name[r] <= 'Z') || (name[r] >= '0' && name[r] <= '7')) {
			previousspace = 0;r++;continue;
		}
		if (name[r] == ' ') {
			if (previousspace == 1) {
				modulebroken = MODULEFAIL_NameInvalid;return;
			}
			if (r == strlen(name)-1) {
				modulebroken = MODULEFAIL_NameInvalid;return;
			}
			previousspace = 1;
			r++;continue;
		}
		modulebroken = MODULEFAIL_NameInvalid;return;
	}
	if (reportedname != NULL) {
		modulebroken = MODULEFAIL_NameTwice;return;
	}
	reportedname = malloc(strlen(name)+1);
	if (reportedname == NULL) {
		modulebroken = MODULEFAIL_MallocFailed;return;
	}
	strcpy(reportedname,name);
}

void weircd_infoq_ReportVersion(int major, int minor, int revision) {
	if (modulebroken > 0) {return;}
	if (module_AccessAllowed(MODAREA_INFOQ) != 0) {return;}
	if (major < 0 || minor < 0 || revision < 0) {
		modulebroken = MODULEFAIL_VersionInvalid;
		return;
	}
	if (major == 0 && minor == 0 && revision == 0) {
		modulebroken = MODULEFAIL_VersionInvalid;
		return;
	}
	if (vmajor != 0 || vminor != 0 || vrevision != 0) {
		modulebroken = MODULEFAIL_VersionTwice;
		return;
	}
	vmajor = major;vminor = minor;vrevision = revision;
}

void weircd_infoq_ReportComment(const char* comment) {
	if (modulebroken > 0) {return;}
	if (module_AccessAllowed(MODAREA_INFOQ) != 0) {return;}
	if (strlen(comment) <= 0 || strlen(comment) >= 150) {
		modulebroken = MODULEFAIL_CommentInvalid;
		return;
	}
	if (reportedcomment != NULL) {
		modulebroken = MODULEFAIL_CommentTwice;
		return;
	}
	reportedcomment = malloc(strlen(comment)+1);
	if (reportedcomment == NULL) {
		modulebroken = MODULEFAIL_MallocFailed;
		return;
	}
	strcpy(reportedcomment,comment);
}

void weircd_infoq_ReportAuthor(const char* names) {
	if (modulebroken > 0) {return;}
	if (module_AccessAllowed(MODAREA_INFOQ) != 0) {return;}
	if (strlen(names) <= 0 || strlen(names) >= 500) {
		modulebroken = MODULEFAIL_AuthorInvalid;
		return;
	}
	if (reportedauthors != NULL) {
		modulebroken = MODULEFAIL_AuthorTwice;
		return;
	}
	reportedauthors = malloc(strlen(names)+1);
	if (reportedauthors == NULL) {
		modulebroken = MODULEFAIL_MallocFailed;
		return;
	}
	strcpy(reportedauthors,names);
}

void weircd_infoq_ReportServicesFunctionality() {
	if (modulebroken > 0) {return;}
	if (module_AccessAllowed(MODAREA_INFOQ) != 0) {return;}
	if (servicesfunc == 1) {
		modulebroken = MODULEFAIL_ReportsFeatureTwice;
		return;
	}
	void* checkptr = modman_GetData(module_Current(),"module_ServicesCOptionPreCheck_Begin");
	if (checkptr == NULL) {
		modulebroken = MODULEFAIL_MissingServicesFunctions;
		return;
	}
	checkptr = modman_GetData(module_Current(),"module_ServicesCOptionPreCheck");
	if (checkptr == NULL) {
		modulebroken = MODULEFAIL_MissingServicesFunctions;
		return;
	}
	checkptr = modman_GetData(module_Current(),"module_ServicesCOptionPreCheck_End");
	if (checkptr == NULL) {
		modulebroken = MODULEFAIL_MissingServicesFunctions;
		return;
	}
	checkptr = modman_GetData(module_Current(),"module_ServicesStartup");
	if (checkptr == NULL) {
		modulebroken = MODULEFAIL_MissingServicesFunctions;
		return;
	}
	checkptr = modman_GetData(module_Current(),"module_ServicesShutdown");
	if (checkptr == NULL) {
		modulebroken = MODULEFAIL_MissingServicesFunctions;
		return;
	}
	checkptr = modman_GetData(module_Current(),"module_NewOptionSet");
	if (checkptr == NULL) {
		modulebroken = MODULEFAIL_MissingServicesFunctions;
		return;
	}
	servicesfunc = 1;
}
void weircd_infoq_ReportLinkingFunctionality() {
	if (modulebroken > 0) {return;}
	if (module_AccessAllowed(MODAREA_INFOQ) != 0) {return;}
	if (linkingfunc == 1) {
		modulebroken = MODULEFAIL_ReportsFeatureTwice;
		return;
	}
	void* checkptr = modman_GetData(module_Current(),"module_PassiveLinkAt");
	if (checkptr == NULL) {
		modulebroken = MODULEFAIL_MissingLinkingFunctions;
		return;
	}
	/*checkptr = modman_GetData(module_Current(),"module_ServicesCOptionPreCheck");
	if (checkptr == NULL) {
		modulebroken = MODULEFAIL_MissingLinkingFunctions;
		return;
	}
	checkptr = modman_GetData(module_Current(),"module_ServicesCOptionPreCheck_End");
	if (checkptr == NULL) {
		modulebroken = MODULEFAIL_MissingLinkingFunctions;
		return;
	}
	checkptr = modman_GetData(module_Current(),"module_ServicesStartup");
	if (checkptr == NULL) {
		modulebroken = MODULEFAIL_MissingLinkingFunctions;
		return;
	}
	checkptr = modman_GetData(module_Current(),"module_ServicesShutdown");
	if (checkptr == NULL) {
		modulebroken = MODULEFAIL_MissingLinkingFunctions;
		return;
	}
	checkptr = modman_GetData(module_Current(),"module_NewOptionSet");
	if (checkptr == NULL) {
		modulebroken = MODULEFAIL_MissingLinkingFunctions;
		return;
	}*/
	linkingfunc = 1;
}
void weircd_infoq_ReportLinkingFunctionalityWithSID() {
	if (modulebroken > 0) {return;}
	weircd_infoq_ReportLinkingFunctionality();
	if (modulebroken > 0) {return;}
	linkingwithsid = 1;
}
char datadirbuf[500];
const char* weircd_GetDataDir() {
	strncpy(datadirbuf,datadir,499);datadirbuf[499] = 0;
	#ifndef WIN
	if (strendswith(datadirbuf,"/") != 0 && strlen(datadirbuf) > 0) {stradd(datadirbuf,"/",499);}
	#endif
	#ifdef WIN
	if (strendswith(datadirbuf,"\\") != 0 && strlen(datadirbuf) > 0) {stradd(datadirbuf,"\\",499);}
	#endif
	return datadirbuf;
}
char helpchanbuf[500];
const char* weircd_GetHelpChan() {
	return config_StrVal("HelpChan");
}
char networknamebuf[100];
const char* weircd_GetNetworkName() {
	strcopy(networknamebuf,config_StrVal("Network"),99);
	return networknamebuf;
}
char svnamebuf[100];
const char* weircd_GetServerName() {
	strcopy(svnamebuf,config_StrVal("ServerName"),99);
	return svnamebuf;
}
char svdescbuf[200];
const char* weircd_GetServerDescription() {
	strcopy(svdescbuf,config_StrVal("ServerDesc"),199);
	return svdescbuf;
}
int weircd_user_IsOpInChan(const char* user, const char* channel) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {return -1;}
	if (udat(u)->pleaseremove == 1) {return -1;}
	CHANNEL c = chan_FindByName(channel);
	if (c == NOCHAN) {return -1;}
	if (isoponchan(u,c) == 0) {
		return 0;
	}
	return -1;
}
int weircd_user_IsIRCOp(const char* user) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {return -1;}
	if (udat(u)->pleaseremove == 1) {return -1;}
	if (udat(u)->isircop == 1) {return 0;}
	return -1;
}
int weircd_user_IsServicesOp(const char* user) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {return -1;}
	if (udat(u)->pleaseremove == 1) {return -1;}
	if (udat(u)->isircop != 1) {return -1;}
	if (udat(u)->opflags.flag_services == 1) {return 0;}
	return -1;
}
unsigned long weircd_user_GetNickTS(const char* user) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {return 0;}
	return udat(u)->lastnickchange;
}
int weircd_user_GetTimeSinceNickChange(const char* user) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {return 0;}
	if (udat(u)->lastnickchange > 0) {
		return (int)((unsigned long long)ts()-udat(u)->lastnickchange);
	}else{
		return 0;
	}
}
const char* weircd_user_GetLoginAccountName(const char* user) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {return NULL;}
	return udat(u)->accountname;
}
const char* weircd_user_GetIp(const char* user) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {return NULL;}
	return udat(u)->ip;
}
const char* weircd_user_GetHost(const char* user) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {return NULL;}
	return udat(u)->host;
}
const char* weircd_user_GetUserid(const char* user) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {return NULL;}
	return udat(u)->suserid;
}
char umodesbuf[50];
const char* weircd_user_GetUmodes(const char* user) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {strcpy(umodesbuf,"");return umodesbuf;}
	irc_ModesForUser(u,umodesbuf,50);
	return umodesbuf;
}
unsigned long long weircd_GetCurrentTime() {
	return ts();
}
int weircd_channel_Exists(const char* channel) {
	CHANNEL c = chan_FindByName(channel);
	if (c == NOCHAN) {return -1;}
	return 0;
}
 
const char* weircd_channel_GetTopic(const char* channel) {
	CHANNEL c = chan_FindByName(channel);
	if (c == NOCHAN) {return NULL;}
	if (cdat(c)->topic_settime > 0) {//gibtn topic
		return cdat(c)->topic;
	}
	return NULL;
}
int weircd_user_IsOn(const char* user) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {return -1;}
	return 0;
}
int weircd_user_IsInChan(const char* user, const char* channel) {
	USER u = usr_FindByNick(user);
	if (u == NOUSER) {return -1;}
	if (udat(u)->pleaseremove == 1) {return -1;}
	CHANNEL c = chan_FindByName(channel);
	if (c == NOCHAN) {return -1;}
	if (usr_IsUserInChan(u,c) == 0) {
		return 0;
	}
	return -1;
}
void* apicycleptr = NULL;
int apicycleglobal = 0;
void weircd_user_CycleAll_SkipGlobals() {
        while (apicycleptr != NULL) {
                if (udat(apicycleptr)->hopcount > 1) {
                        apicycleptr = linkedlist_NextItem(apicycleptr);
                        continue;
                }
                break;
        }
}
void weircd_user_CycleAll_Init(int global) {
        apicycleptr = linkeduserlist;
        apicycleglobal = global;
        if (apicycleglobal == 0) {
                weircd_user_CycleAll_SkipGlobals();
        }
}
 
const char* weircd_user_CycleAll_Do() {
         apicycleptr = linkedlist_NextItem(apicycleptr);
         if (apicycleglobal == 0) {
                weircd_user_CycleAll_SkipGlobals();
        }
        if (apicycleptr == NULL) {return NULL;}
        return udat(apicycleptr)->nickname;
}
const char* weircd_user_GetServer(const char* user) {
        USER u = usr_FindByNick(user);
        if (u == NOUSER) {return NULL;}
        SERVER c = udat(u)->homeserver;
        if (c != NOSERVER) {
                return server_GetName(c);
        }
        return NULL;
}
int weircd_user_GetHopcount(const char* user) {
        USER u = usr_FindByNick(user);
        if (u == NOUSER) {return 1;}
        return udat(u)->hopcount;
}
const char* weircd_user_GetRealname(const char* user) {
        USER u = usr_FindByNick(user);
        if (u == NOUSER) {return NULL;}
        return udat(u)->realname;
}