//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"

// This implements modules/weircd_api_fakeusers.h functions

#define PROCEEDBUF 1024
struct fakeuser {
	int publichandle;
	void* userreference;
	int mustbehandled;
	int proceedbufsize;
	char* proceedbuf;
	void* readbuf; //manually allocate this around as needed (or maybe in chnks?)
	               //make sure it stays there even _after_ userreference went NULL
		       //so a module can still read the read stack contents left even
		       //when the connection is in reality already gone (before it
		       //receives a notify of the connection being terminated then)
};
struct fakeuserlist {
	void* fakeuserlist;
	void* parentmodule;
};
void* listoffakeuserlists = NULL;

void* FakeUsers_GetFakeUserByHandle(int HANDLE, void* moduleentry) {
	struct fakeuserlist* l = linkedlist_GetData(moduleentry);
	void* item = l->fakeuserlist;
	while (item != NULL) {
		struct fakeuser* f = linkedlist_GetData(item);
		if (f->publichandle == HANDLE) {
			return item;
		}
		item = linkedlist_NextItem(item);
	}
	return NULL;
}
USER FakeUsers_GetUserPtr(int handle, void* moduleentry) {
	void* fuser = FakeUsers_GetFakeUserByHandle(handle, moduleentry);
	if (fuser == NULL) {return NULL;}
	struct fakeuser* f = linkedlist_GetData(fuser);
	return f->userreference;
}
void* FakeUsers_GetModuleFakeEntryByPtr(void* ptr) {
	void* item = listoffakeuserlists;
	while (item != NULL) {
		struct fakeuserlist* l = linkedlist_GetData(item);
		if (l->parentmodule == ptr) {
			return item;
		}
		item = linkedlist_NextItem(item);
	}
	return NULL;
}
void* FakeUsers_GetModuleFakeEntryByName(const char* name) {
	void* mptr = modman_GetPtrByName(name);
	if (mptr == NULL) {return NULL;}
	return FakeUsers_GetModuleFakeEntryByPtr(mptr);
}
int obtainfreefakeuserhandle(void* module) {
	void* m = FakeUsers_GetModuleFakeEntryByPtr(module);
	if (m == NULL) {return 0;}
	int i = 0;
	struct fakeuserlist* l = linkedlist_GetData(m);
	void* item = l->fakeuserlist;
	while (item != NULL) {
		struct fakeuser* f = linkedlist_GetData(item);
		if (f->publichandle >= i) {i = f->publichandle+1;}
		item = linkedlist_NextItem(item);
	}
	return i;
}
void _deletefakeuserbypointer(void* ptr, void* moduleentry) {
	struct fakeuser* f = linkedlist_GetData(ptr);
	if (f->userreference != NULL) {
		udat(f->userreference)->fakeuser = 0;
		udat(f->userreference)->socket = -1;
		userQuit(f->userreference,USERQUIT_EOF);//user will become terminated and we are no longer responsible for it
	}
	if (f->proceedbuf != NULL) {free(f->proceedbuf);}
	struct fakeuserlist* l = linkedlist_GetData(moduleentry);
	linkedlist_RemoveItem(&l->fakeuserlist, ptr);
	/*if (l->fakeuserlist == NULL) { //EVIL don't do this here
		
	}*/
}
void wipeoutfakeuserlistmoduleentry(void* entry) {
	struct fakeuserlist* l = linkedlist_GetData(entry);
	while (l->fakeuserlist != NULL) {
		_deletefakeuserbypointer(l->fakeuserlist, entry);
	}
	linkedlist_RemoveItem(&listoffakeuserlists, entry);
}
void wipeoutmodule(void* module) {
	void* item = listoffakeuserlists;
	while (item != NULL) {
		struct fakeuserlist* l = linkedlist_GetData(item);
		if (l->parentmodule == module) {
			wipeoutfakeuserlistmoduleentry(item);
			return;
		}
		item = linkedlist_NextItem(item);
	}
}
void* createmodulefakeentry(void* module) {
	struct fakeuserlist l;
	memset(&l,0,sizeof(l));
	l.parentmodule = module;
	return linkedlist_Append(&listoffakeuserlists,&l,sizeof(l));
}
void FakeUsers_Delete(void* module, void* fakeuser) {
	void* mptr = FakeUsers_GetModuleFakeEntryByName(module);
	if (mptr == NULL || fakeuser == NULL) {return;}
	_deletefakeuserbypointer(fakeuser,mptr);
}
void FakeUsers_FreeByPtr(void* ptr) {
	void* mod = FakeUsers_GetModuleFakeEntryByName(module_Current());
	if (mod == NULL) {return;}
	struct fakeuserlist* l = linkedlist_GetData(mod);
	void* item = l->fakeuserlist;
	while (item != NULL) {
		if (item == ptr) {
			_deletefakeuserbypointer(item,mod);
			return;
		}
		item = linkedlist_NextItem(item);
	}
}
void weircd_fakeuser_Free(int handle) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return;	
	}
	
}
void FakeUsers_DeleteAll(void* module) {
	printf("deleting all fake users\n");
	void* item = listoffakeuserlists;
	while (item != NULL) {
		struct fakeuserlist* l = linkedlist_GetData(item);
		if (l->parentmodule == module) {
			while (l->fakeuserlist != NULL) {
				_deletefakeuserbypointer(l->fakeuserlist,item);
			}
			return;
		}
		item = linkedlist_NextItem(item);
	}
}
typedef void (*fakeuserrecvfunc)(int handle, const char* data);
void FakeUsers_Send(USER uhandle, const char* stuff) {
	if (uhandle == NOUSER) {return;}
	int i = 0;
	void* item = listoffakeuserlists;
	while (item != NULL) {
		struct fakeuserlist* l = linkedlist_GetData(item);
		void* item2 = l->fakeuserlist;
		while (item2 != NULL) {
			struct fakeuser* f = linkedlist_GetData(item2);
			if (f->userreference == uhandle) {
				const char* p = modman_GetNameByPtr(l->parentmodule);
				fakeuserrecvfunc rfunc = modman_GetData(p,"module_FakeUserRecv");
				module_Switched(p);module_SetArea(MODAREA_GENERICFUNC);
				rfunc(f->publichandle,stuff);
				if (module_AccessViolationHappened() == 0) {
					modlist_SafeShutdown(p);
				}
				return;
			}
			i++;
			item2 = linkedlist_NextItem(item2);
		}
		item = linkedlist_NextItem(item);
	}
}
static int FakeUsers_AddToProceedBuf(int uhandle, void* module, const char* stuff) {
	void* mentry = FakeUsers_GetModuleFakeEntryByPtr(module);
	if (mentry == NULL) {return -1;}
	void* fuser = FakeUsers_GetFakeUserByHandle(uhandle, mentry);
	struct fakeuser* f = linkedlist_GetData(fuser);
	if (f->userreference == NULL) {
		return -1;//user is disconnected
	}
	if (f->proceedbuf == NULL) {
		f->proceedbufsize = PROCEEDBUF;
		f->proceedbuf = malloc(f->proceedbufsize);
		if (f->proceedbuf == NULL) {f->proceedbufsize = 0;return 0;}
		strcpy(f->proceedbuf,"");
	}
	int len = strlen(stuff)+1;
	if (len > 550) {len = 550;}
	if (strlen(f->proceedbuf)+len >= f->proceedbufsize) {
		int oldsize = f->proceedbufsize;
		f->proceedbufsize *= 2;
		void* n = realloc(f->proceedbuf, f->proceedbufsize);
		if (n == NULL) {
			f->proceedbufsize = oldsize;
			return 0;
		}
		f->proceedbuf = n;
	}
	stradd(f->proceedbuf,stuff,f->proceedbufsize-1);
	stradd(f->proceedbuf,"\n",f->proceedbufsize-1);
	return 0;
}
int weircd_fakeuser_Send(int handle, const char* line) {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	void* p = modman_GetPtrByName(module_Current());
	if (p == NULL) {return -1;}
	return FakeUsers_AddToProceedBuf(handle, p, line);
}
void FakeUser_ProceedStuff(void* fuser) {
	char line[513];
	struct fakeuser* f = linkedlist_GetData(fuser);
	if (f->proceedbuf == NULL) {return;}
	if (strlen(f->proceedbuf) <= 0 || f->userreference == NULL) {return;}
	while (charpos(f->proceedbuf, '\n') > 0) {
		strcopy(line,f->proceedbuf,512);
		if (charpos(line,'\n') > 0) {line[charpos(line,'\n')] = 0;}
		trimright(line);
		cut(f->proceedbuf,charpos(f->proceedbuf,'\n')+1);
		if (strlen(line) > 0) {
			//printf("Proceeding: %s\n",line);
			userman_SplitCMD(f->userreference, line);
		}
	}
	strncpy(line,f->proceedbuf,512);line[512] = 0;trimright(line);
	if (strlen(line) > 0) {
		//printf("Proceeding: %s\n",line);
		userman_SplitCMD(f->userreference, line);
	}
	strcpy(f->proceedbuf,"");
}
void FakeUsers_ProceedAll() {
	void* item = listoffakeuserlists;
	while (item != NULL) {
		struct fakeuserlist* l = linkedlist_GetData(item);
		void* item2 = l->fakeuserlist;
		while (item2 != NULL) {
			void* nextitem2 = linkedlist_NextItem(item2);
			FakeUser_ProceedStuff(item2);
			item2 = nextitem2;
		}
		item = linkedlist_NextItem(item);
	}
	return;
}
void* FakeUsers_Create(void* module) {
	void* m = FakeUsers_GetModuleFakeEntryByPtr(module);
	if (m == NULL) {
		m = createmodulefakeentry(module);
		if (m == NULL) {return NULL;}
	}
	struct fakeuserlist* l = linkedlist_GetData(m);
	struct fakeuser fuser;
	memset(&fuser,0,sizeof(fuser));
	fuser.userreference = usr_AddUser(-1,-1,NULL,0);
	fuser.publichandle = -1;
	if (fuser.userreference == NOUSER) {
		return NULL;
	}
	void* item = linkedlist_Append(&l->fakeuserlist,&fuser,sizeof(fuser));
	return item;
}
void* weircd_fakeuser_Create_byptr(void* module) {
	return FakeUsers_Create(module);
}
int weircd_fakeuser_Create() {
	if (module_AccessAllowed(MODAREA_GENERICFUNC) != 0) {
		return -1;	
	}
	void* ptr = modman_GetPtrByName(module_Current());
	if (ptr == NULL) {
		return -1;
	}
	void* newuser = weircd_fakeuser_Create_byptr(ptr);
	if (newuser == NULL) {return -1;}
	int handle = obtainfreefakeuserhandle(ptr);
	struct fakeuser* fu = linkedlist_GetData(newuser);
	fu->publichandle = handle;
	return handle;
}
void FakeUsers_DropUserReference(void* connection) {
	void* item = listoffakeuserlists;
	while (item != NULL) {
		struct fakeuserlist* flist = linkedlist_GetData(item);
		void* item2 = flist->fakeuserlist;
		while (item2 != NULL) {
			struct fakeuser* u = linkedlist_GetData(item2);
			if (u->userreference == connection) {
				u->userreference = NULL;
				return;
			}
			item2 = linkedlist_NextItem(item2);
		}
		item = linkedlist_NextItem(item);
	}
}
void FakeUsers_ShutdownForModule(const char* name) {
	void* ptr = modman_GetPtrByName(name);
	if (ptr == NULL) {return;}
	wipeoutmodule(ptr);
}