//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

/* moduleslist.c - Extended module state manager */
/* --------------------- */
/* This module manager holds a list of the current state of loaded modules
   and their activated functionalities and can be used to construct a
   target state (config state) to which a change can be requested. It
   will then try to reach that state as much as possible.
   It will load, unload and change things to change to the wanted target stat
   as much as possible.
   This is much more highlevel than modulesmanager.c which simply manages
   the loaded libs and can load/unload them on request (without caring about
   what they are used for and what should be called to properly shutdown
   a module so it can free its data). */

/* Runtime changes: */
void modlist_SafeShutdown(const char* name); /* Request a safe (immediate) shutdown of a module, e.g. when it did something wrong or if there is a runtime request by a server admin */
void modules_ShutdownAll(); /* Use before shutting down the server */

/* Maintain the target/config state: */
void modlist_Clear(); /* Obtain a new clear target state that can be filled */
void modlist_AddAsServices(const char* name); /* Add a module to be loaded up as services module */
void modlist_AddServicesOption(const char* name, const char* option, const char* value); /* Add services options to be used */

/* Config independent realtime request of a module for linking */
int modlist_LoadForLinking(const char* name); /* returns -1 on failure */

/* Apply changes: */
void modlist_ApplyChanges(); /* Applies changes. If any module is highly corrupt the server might now have a good opportunity to crash ;-) */

/* Other interesting informational stuff: */
int modlist_Check(const char* name); /* Check if a module is currently loaded */
int modlist_Print
    (USER uid,
     const char* format); /* Send a list of modules to the user by sending
     the format string and replacing the first  %s with the module name and the
     second occuring %s with its type.
     The format should represent a valid PRIVMSG or NOTICE or something comparable
     that is a valid IRC event to be sent to the user. */