//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2009 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

/* modulesmanager.c - Simple managed module loader */
/* --------------------- */
/* Loads libs and modules and keeps them in a list with name
   and pointer so they can be easily found and unloaded as a whole
   without forgetting a module or loading it twice. */

/* Load/unload modules: */
int modman_Load(const char* modname); /* Load a module by name (NO file extension or full path to the modules dir!)
                                         Returns 0 on success, <0 when failing. */
void modman_Unload(const char* modname); /* Unload a module by the name that was used when loading it */

/* Do stuff with pointer handles */
//WARNING: do NEVER submit an invalid pointer, it WILL segfault (or corrupt things badly)
void* modman_GetPtrByName(const char* modname); /* Obtain the pointer handle of a loaded module */
const char* modman_GetNameByPtr(void* ptr); /* Obtain the name of a module by a valid pointer handle */
void modman_UnloadByPtr(void* ptr); /* Unload a module by pointer handle */

/* Access stuff inside the module by resolving symbols */
void* modman_GetData(const char* modname, const char* data); /* Resolve a symbol named by "data" of a loaded module "modname" and return a pointer to it or NULL */
void* modman_GetDataByPtr(void* modptr, const char* data);


