//    WeIRCd IRC server software - sub file
//    Copyright (C) 2007-2010 Jonas Thiem
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License v3 as published
//    by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "weircd.h"
#include <ctype.h>

#ifndef WIN
void strlwr(char *s) {
	while (*s) {
		*s = tolower(*s);
		s++;
	}
}
#endif
void strupper(char *s) {
	while (*s) {
		*s = toupper(*s);
		s++;
	}
}
int issimplenumber(const char* s) {
	if (strlen(s) <= 0) {return -1;}
	while (*s != '\0') {
		if (*s < '0' || *s > '9') {return -1;}
		s++;
	}
	return 0;
}

int containsmixedcaseletters(const char* s) {
	int lowercase = 0;
	int uppercase = 0;
	while (*s != '\0') {
		if (islower(*s)) {
			lowercase = 1;
		}else{
			uppercase = 1;
		}
		if (lowercase == 1 && uppercase == 1) {
			return 1;
		}
		s++;
	}
	return 0;
}
int containsnumbers(const char* s) {
	while (*s != '\0') {
		if (*s >= '0' && *s <= '9') {return 0;}
		s++;
	}
	return -1;
}

int stringisinstring(const char* longstring, const char* shortstring) {
	const char *str = longstring;
	const char *substr = shortstring;
	while (*str) {
		if (*str == *substr || tolower(*substr) == tolower(*str)) {
			substr++;
			if(*substr == '\0') {
				return 0;
			}
		}else{
			substr = shortstring;
		}
		str++;
	}
	return -1;
}


void stradd(char* dest, const char* src, int len) {
	int addlen = len;
	addlen -= strlen(dest);
	strncat(dest,src,addlen);
}
void strcopy(char* dest, const char* src, int len) {
	strncpy(dest,src,len);dest[len] = 0;
}
int charpos(const char* stringle, char letter) {
	char *str;
	str = strchr(stringle, letter);
	if(str) {
		return str - stringle;
	}
	return -1;
}

int charposfromright(const char* stringle, char letter) {
	char *str;
	str = strrchr(stringle, letter);
	if(str) {
		return str - stringle;
	}
	return -1;
}

void cut(char* str, int count) {
	if (count <= 0) {return;}
	int r = strlen(str);
	if (count >= r) {strcpy(str,"");return;}
	memmove(str, str+count, r+1);
}
int casecomparechars(char c1, char c2) {
	if(c1 == c2 || tolower(c1) == tolower(c2)) {
		return 0;
	}
	return -1;
}
int strcasecmplen(const char* str1, const char* str2, unsigned int len) {
	unsigned int r = 0;
	while (r < len && r < strlen(str1) && r < strlen(str2)) {
		if (casecomparechars(str1[r],str2[r]) != 0) {
			return -1;
		}
		r++;
	}
	return 0;
}

int strstw(const char* bigstring, const char* substr) {
	const char *str = bigstring;
	while (*str) {
		if (*str == *substr || tolower(*substr) == tolower(*str)) {
			substr++;
			if (*substr == '\0') {
				return 0;
			}
		}else{
			return -1;
		}
		str++;
	}
	if (*substr) {return -1;}
	return 0;
}

int strendswith(const char* bigstring, const char* endstring) {
	unsigned int len = strlen(bigstring);
	unsigned int len2 = strlen(endstring);
	if (len2 > len) {return -1;}
	const char *str = bigstring+(len-len2);
	const char *substr = endstring;
	while(*str) {
		if (*str == *substr || tolower(*substr) == tolower(*str)) {
			substr++;
			if (*substr == '\0') {
				return 0;
			}
		}else{
			return -1;
		}
		str++;
	}
	return -1;
}


int onlyspaces(const char* vstring) {
	while (*vstring != 0) {
		if (*vstring != 32 && *vstring != 9) {return -1;}
		vstring++;
	}
	return 0;
}
void trimright(char* vstring) {
	if (vstring == NULL) {return;}
	unsigned int len = strlen(vstring);
	if (len < 1) {return;}
	while (vstring[len-1] == 32 || vstring[len-1] == 9 || vstring[len-1] == '\n'
		|| vstring[len-1] == '\r') {
		vstring[len-1] = 0;//cut last char because it is a space char
		len--;
		if (len <= 0) {return;}
	}
}
void trimleft(char* str) {
	if (str == NULL) {return;}
	unsigned int i = 0;
	while (i < strlen(str)) {
		if (str[i] != ' ' && str[i] != 9 && str[i] != '\n') {
			break;
		}
		i++;
	}
	if (i > 0) {
		cut(str,i);
	}
}
void cutonfirstspace(char* str) {
	if (str == NULL) {return;}	
	while (*str) {
		if (*str == 32 || *str == 9 || *str == '\n') {
			*str = 0;
			return;
		}
		str++;
	}
}
void fixlinebreaks(char* buf, int len) {
	/* This will replace a single \r with \n and \r\n with \n\040 (a leading space char).
	   The space char is there to increase performance by not shifting the whole following output around to
	   get rid of that byte in between. */
	int z = 0;
	while (z < len-1) {
		if (buf[z] == '\r') {
			buf[z] = '\n';
			if (buf[z+1] == '\n') {
				buf[z+1] = 32;
			}
		}
		z += 1;
	}
}
void addtobuf(char* buf, char byte, unsigned int maxlen) {
	unsigned int len = strlen(buf);
	if (len < maxlen-1) {
		buf[len+1] = 0;
		buf[len] = byte;
		return;
	}
}
int isipoctetchar_check(char c, int type) {
	if (c >= '0' && c <= '9') {return 0;}
	if (c >= 'a' && c <= 'f') {return 0;}
	if (c >= 'A' && c <= 'F') {return 0;}
	return -1;
}
int isip_check(const char* str, int type) {
	char val[] = "";
	int octet = 1;int octetlen = 0;
	int previousiscolon = 1;
	int doublecolon = 0;
	char colonchar = '.';
	if (type == 1) { /* the colon type we want to look for */
		colonchar = ':';
		previousiscolon = 0;//we may start with a colon when using ipv6
	}
	int len = strlen(str);
	int r = 0;
	while (r < len) {
		if (isipoctetchar_check(str[r],type) != 0) { /* are we still in an octet? probably not -> */
			if (str[r] == colonchar && previousiscolon == 0) { /* check if it's a proper separator ending an octet */
				if (r == 0) {//IPv6: we *need* to be at a double colon. a single colon at start is forbidden
					if (len < 2) {return -1;}
					if (str[r+1] != colonchar) {return -1;}
					previousiscolon = 1;
					octet = 1;r++;continue;
				}
				octet++;octetlen = 0;
				if (octet > 4 && type != 1) {return -1;}
				if (octet > 8 && type == 1 && r > 0) {return -1;}
				if (type != 1) {
					if (atoi(val) > 255) {return -1;}
					strcpy(val,"");
				}
				previousiscolon = 1;
				r++;continue;
			}
			if (str[r] == colonchar && previousiscolon == 1 && doublecolon == 0 && type == 1) {
				/* check if it's a proper double separator in ipv6 */
				doublecolon = 1;
				r++;continue;
			}
			return -1;
		}else{
			previousiscolon = 0;
			octetlen++;
			if (octetlen > 3 && type != 1) {return -1;}
			if (octetlen > 4 && type == 1) {;return -1;}
			if (type != 1) {
				val[strlen(val)+1] = 0;
				val[strlen(val)] = str[r];
			}
		}
		r++;
	}
	if (type != 1) {
		if (atoi(val) > 255) {return -1;}
	}
	if (octetlen <= 0) {octet--;}
	if ((octet < 3 || previousiscolon == 1) && type != 1) {return -1;}
	if (octet < 8 && type == 1 && (doublecolon == 0 || octet <= 0)) {return -1;}
	return 0;
}
int isipv4ip(const char* str) {
	return isip_check(str,0);
}
int isipv6ip(const char* str) {
	return isip_check(str,1);
}
int iptest(const char* str, int type, int expected) {
	int r = isip_check(str,type);
	printf("%d-(%d)",r,expected);
	if (expected == 0 && r != 0) {
		printf("[%s:FAIL]\n",str);return 0;
	}
	if (r == 0 && expected != 0) {
		printf("[%s:FAIL]\n",str);return 0;
	}
	printf("[%s:SUCCESS]\n",str);
	return -1;
}